# CSS Form Styling Module Level 1

## A Collection of Interesting Ideas, 1 January 1970

This version:
  https://drafts.csswg.org/css-forms/

Not Ready For Implementation

This spec is not yet ready for implementation. It exists in this repository to record the ideas and promote discussion.

Before attempting to implement this spec, please contact the editors.

To the extent possible under law, the editors have waived all copyright and related or neighboring rights to this work. In addition, as of 1 January 1970, the editors have made this specification available under the Open Web Foundation Agreement Version 1.0, which is available at http://www.openwebfoundation.org/legal/the-owf-1-0-agreements/owfa-1-0. Parts of this work may be from another specification document. If so, those parts are instead covered by the license of that specification document.

## Abstract

This document is currently a loosely-structured set of ideas and inspiration for CSS form styling. It is not an implementable standard. Do not look at this as anything but a collection of ideas.

## 1. Basic Styling Proposals

This section sketches a few proposals for solving the form styling problem.

### 1.1. Prototypes

This idea, originally suggested by fantasai, is that we can style a handful of "prototype" elements. Browser UI designers can then take the styling of those elements and extrapolate the design into their own UIs. At minimum, things like text, backgrounds, and borders can be used. At the limit, things like internal padding, border-radius, etc might be used.

    @control button {
      <declaration-list>
    }

    @control input {
      <declaration-list>
    }

    input::selection {
      <declaration-list>
    }

    ...

> You would be able to use styles for:

> - font selection (css-fonts)

> - text decoration (css-text-decor)

> - text layout (css-text)

> - backgrounds & drop-shadow (css-backgrounds)

> - borders & padding (css-backgrounds)

> - filters

> - anything else we or a UA decides seems relevant

> Most form controls, even a calendar widget or clock, are a combination of these three primitives in some way. If the UA is given the styling for these three primitives, and perhaps one or two more it can figure out how to style the rest.

> For example, a calendar widget might have the month, the year, some buttons to move them around, the ability to click into them and edit them directly, and a representation of the days of the month. The selected day is selected. Perhaps the buttons only show up on :hover or :focus -- the UA decides. But it knows that a button should be this particular shade of blue with that particular border-radius and drop-shadow. The calendar might be shown in the colors of the input field, and the selected day in the selection color, and in all ways it will match the way the input fields look in the rest of the page.

> Now, the author can’t decide, for example, the spacing between the year and the month name, or whether the button to change years has a solid arrow or a hollow arrow or a frilly one, and she can’t decide that there should be a black solid half-border between the month and the day field below it with 5px spacing. But the calendar will look like it belongs to the page, and the UA can come up with a different calendar layout when it ships one on a wide but short smart watch where the month and year are better placed on the side without breaking anything.

> The black area is the button color; a very light transparency of it can be the glass color. The rollers are the input colors.

> It’s hard to tell without more context, but for the one on the right, the clock face and the digital readout are @input colors, the highlighted bits are the highlight color, the Done button is the button styling, and the shaded area around the clock face is the same color as the button background.

### 1.2. Inverse System Colors

This idea, originally sketched by Florian and Tab, is to define an abstract set of colors that UI designers can then choose from when coloring their UI.

Tab’s suggested set of colors, from an Android color-extraction API proposal:

- Light/Normal/Dark Vibrant

- Light/Normal/Dark Muted

- Vibrant Complementary (for call-out buttons and such that need to be visually distinct)

(where light ~75% lightness, normal is ~50%, dark is ~25%; vibrant is at least 30% saturation, ideally 100%, and muted is at most 40% saturation, ideally 30%)

- Light/Dark Contrasting Text

- Light/Dark Contrasting Secondary Text

That’s 11 colors, many of which should be drawable from the webpage’s own color scheme. We can auto-gen a bunch of them if you specify at least some of them, like genning the light/dark variants from the "normal-vibrant" color, or automatically setting text colors to white/black as appropriate.

There’s no guarantee that the input UIs will use the colors in the *same way* that the rest of the page does, though.

## 2. Miscellaneous Control-Specific Suggestions

<progress> and <meter> styling

Insert <select> and <datalist> styling proposal and/or whiteboard photo.

### 2.1. Select/Datalist Dropdown

1. Only allow styling if both color and background are set.

2. Option container:

   - backgrounds

   - borders

   - padding

3. `option`

   - padding

   - borders

   - border-collapse?

   - backgrounds

   - display-inside is allowed, automatically blockified

   - not margins, position, float, width, height

   All options are contain:paint and BFCs.

## 3. Input UI Examples

This section catalogues as many input UI examples as we can screenshot, especially on mobile devices where they’re a bit "weirder".

### 3.1. Time Pickers

iOS time picker
Android time picker 1
Android time picker 2

### 3.2. Date Pickers

Android date picker

## Conformance

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

## Index

https://drafts.csswg.org/css-backgrounds-3/#propdef-background Referenced in:

- 2.1. Select/Datalist Dropdown

https://drafts.csswg.org/css-color-4/#propdef-color Referenced in:

- 2.1. Select/Datalist Dropdown

https://html.spec.whatwg.org/multipage/form-elements.html#the-option-element Referenced in:

- 2.1. Select/Datalist Dropdown

### Terms defined by reference

- [css-backgrounds-3] defines the following terms:
  - background
- [css-color-4] defines the following terms:
  - color
- [HTML] defines the following terms:
  - option

## References

### Normative References

[CSS-BACKGROUNDS-3]
  Bert Bos; Elika Etemad; Brad Kemper. CSS Backgrounds and Borders Module Level 3. 17 October 2017. CR. URL: https://www.w3.org/TR/css-backgrounds-3/
[CSS-COLOR-4]
  Tab Atkins Jr.; Chris Lilley. CSS Color Module Level 4. 5 March 2019. WD. URL: https://www.w3.org/TR/css-color-4/
[HTML]
  Anne van Kesteren; et al. HTML Standard. Living Standard. URL: https://html.spec.whatwg.org/multipage/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119

## Issues Index

<progress> and <meter> styling ↵
Insert <select> and <datalist> styling proposal and/or whiteboard photo. ↵